%% function cputest
% This is a simple test to try and identify if Matlab is making correct
% calculations. It was motivated by the not uncommon occurance of Matlab
% using a faulty/wrong BLAS version for the CPU in the case of newer 64-bit
% CPUs. Although it only checks for for errors involving single-precision data, 
% which were very significant, there were also significant but less noticable errors
% in double-precision calculations when calling the wrong BLAS library.
%   It also checks for a more subtle problem concerning a difference in
% single-precision summation, which may be a rounding error on the part of 
% the 64bit version of Matlab.
% 
% USAGE:
%   >> cputest
%
% INPUTS: the file cputest_data.mat should be in the current directory.
%
% SOME NOTES ON TESTS:
%   Failing left division and rank test indicated more serious problems, which only
%   appear to be related to use of the Matlab 64bit Intel BLAS library:
%   libmkl.so and libguide.so
%
%   Summation, multiplication and inverse test are against the results from
%   Matlab2006b, the 32bit version. The summation test was converted to binary and 
%   confirmed by hand, and is partly the reasoning for concluding that the 32bit
%   version is the one producing correct results.
%
% CHANGING YOUR BLAS VERSION
%   the blas library to be used is specified in a file called blas.spec
%   On our Linux 32bit system it is located in 
%   /usr/common/pkgs/matlab-r2006b/bin/glnx86
%
% This is not meant to be anything close to a comprehensive test of Matlab's
% accuracy. It only reflects a number of distinct observed discrepancies. 

% Copyright (C) Toby Fernsler and Scott Makeig, SCCN/UCSD Feb 13, 2007
%                                          
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%% INITIALIZE
function cputest
load('cputest_data.mat');

%% TEST LEFT DIVISION
lastwarn('')
A\weights;
if isempty(lastwarn)
   fprintf('Passed left-division test (mldivide.m).\n');
else
   fprintf('Failed left-division test (mldivide.m), erroneous warning issued.\n');
end

%% TEST RANK FUNCTION
if rank(A) == 30
    fprintf('Passed the rank test.\n');
    fprintf('  Measured rank(A) = %d . Actual rank(A) = 30.\n',rank(A));
else
    fprintf('Failed the rank test.\n');
    fprintf('  Measured rank(A) = %e . Actual rank(A) = 30.\n',rank(A));
end
%% TEST SUMMATION
if sumsumthis == sum(sumthis)
    fprintf('Passed addition test.\n');
    fprintf('  sum(sumthis) = %10.8f . Actual sum(sumthis) = %10.8f .\n', sum(sumthis), sumsumthis);
else
    fprintf('Failed addition test.\n');
    fprintf('  sum(sumthis) = %10.8f . Actual sum(sumthis) = %10.8f .\n', sum(sumthis), sumsumthis);
end

%% TEST MATRIX MULTIPLICATION
if gstarweights == g*weights
    fprintf('Passed matrix multiplication test.\n');
    fprintf('  g*weights = %10.8f . Actual g*weights = %10.8f .\n', g*weights, gstarweights);
else
    fprintf('Failed matrix multiplication test.\n');
    fprintf('  g*weights = %10.8f . Actual g*weights = %10.8f .\n', g*weights, gstarweights);
end

%% TEST INVERSE FUNCTION
if invA == inv(A)
    fprintf('Passed inverse test.\n');
    fprintf('  max(max(abs(eye(30)-A*inv(A)))) = %10.8e\n',max(max(abs(eye(30)-A*inv(A)))) );
    fprintf('  baseline result was %10.8e .\n', errinvA);
else
    fprintf('Failed inverse test.\n');
    fprintf('  max(max(abs(eye(30)-A*inv(A)))) = %10.8e\n',max(max(abs(eye(30)-A*inv(A)))) );
    fprintf('  baseline result was %10.8e .\n', errinvA);
end


